package com.sun.jndi.dns;

import java.util.Iterator;
import java.io.UnsupportedEncodingException;
import com.sun.jndi.toolkit.url.UrlUtil;
import java.util.List;
import java.util.ArrayList;
import java.net.MalformedURLException;
import javax.naming.ConfigurationException;
import sun.net.dns.ResolverConfiguration;
import javax.naming.NamingException;
import javax.naming.Context;
import java.util.Hashtable;
import javax.naming.spi.InitialContextFactory;

public class DnsContextFactory implements InitialContextFactory
{
    private static final String DEFAULT_URL = "dns:";
    private static final int DEFAULT_PORT = 53;
    
    public DnsContextFactory() {
    }
    
    @Override
    public Context getInitialContext(Hashtable<?, ?> hashtable) throws NamingException {
        if (hashtable == null) {
            hashtable = new Hashtable(5);
        }
        return urlToContext(getInitCtxUrl(hashtable), hashtable);
    }
    
    public static DnsContext getContext(String s, String[] array, Hashtable<?, ?> hashtable) throws NamingException {
        return new DnsContext(s, array, hashtable);
    }
    
    public static DnsContext getContext(String s, DnsUrl[] array, Hashtable<?, ?> hashtable) throws NamingException {
        String[] serversForUrls = serversForUrls(array);
        DnsContext context = getContext(s, serversForUrls, hashtable);
        if (platformServersUsed(array)) {
            context.setProviderUrl(constructProviderUrl(s, serversForUrls));
            return context;
        }
        return context;
    }
    
    public static boolean platformServersAvailable() {
        return !filterNameServers(ResolverConfiguration.open().nameservers(), true).isEmpty();
    }
    
    private static Context urlToContext(String s, Hashtable<?, ?> hashtable) throws NamingException {
        DnsUrl[] fromList;
        try {
            fromList = DnsUrl.fromList(s);
        }
        catch (MalformedURLException ex) {
            throw new ConfigurationException(ex.getMessage());
        }
        if (fromList.length == 0) {
            throw new ConfigurationException("Invalid DNS pseudo-URL(s): " + s);
        }
        String domain = fromList[0].getDomain();
        int i = 1;
        while (i < fromList.length) {
            if (!domain.equalsIgnoreCase(fromList[i].getDomain())) {
                throw new ConfigurationException("Conflicting domains: " + s);
            }
            ++i;
        }
        return getContext(domain, fromList, hashtable);
    }
    
    private static String[] serversForUrls(DnsUrl[] array) throws NamingException {
        if (array.length == 0) {
            throw new ConfigurationException("DNS pseudo-URL required");
        }
        ArrayList<Object> list = new ArrayList();
        for (int i = 0; i < array.length; ++i) {
            String host = array[i].getHost();
            int port = array[i].getPort();
            if (host == null && port < 0) {
                List<String> filterNameServers = filterNameServers(ResolverConfiguration.open().nameservers(), false);
                if (!filterNameServers.isEmpty()) {
                    list.addAll(filterNameServers);
                    continue;
                }
            }
            if (host == null) {
                host = "localhost";
            }
            list.add((port < 0) ? host : (host + ":" + port));
        }
        return (String[])list.toArray(new String[list.size()]);
    }
    
    private static boolean platformServersUsed(DnsUrl[] array) {
        if (!platformServersAvailable()) {
            return false;
        }
        int i = 0;
        while (i < array.length) {
            if (array[i].getHost() == null && array[i].getPort() < 0) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    private static String constructProviderUrl(String s, String[] array) {
        String string = "";
        if (!s.equals(".")) {
            try {
                string = "/" + UrlUtil.encode(s, "ISO-8859-1");
            }
            catch (UnsupportedEncodingException ex) {}
        }
        StringBuffer o = new StringBuffer();
        for (int i = 0; i < array.length; ++i) {
            if (i > 0) {
                o.append(' ');
            }
            o.append("dns://").append(array[i]).append(string);
        }
        return o.toString();
    }
    
    private static String getInitCtxUrl(Hashtable<?, ?> hashtable) {
        String s = (String)hashtable.get("java.naming.provider.url");
        return (s != null) ? s : "dns:";
    }
    
    private static List<String> filterNameServers(List<String> list, boolean b) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager == null || list == null || list.isEmpty()) {
            return list;
        }
        ArrayList<String> list2 = new ArrayList();
        for (String s : list) {
            int index = s.indexOf(58, s.indexOf(93) + 1);
            int port = (index < 0) ? 53 : Integer.parseInt(s.substring(index + 1));
            String host = (index < 0) ? s : s.substring(0, index);
            try {
                securityManager.checkConnect(host, port);
                list2.add(s);
                if (b) {
                    return list2;
                }
                continue;
            }
            catch (SecurityException ex) {}
        }
        return list2;
    }
}
